#include "../Framework/Framework.h"
#include "../Framework/Util/T3dLoader.h"
#include "DepthPolygon.h"

class MainApp : public OpenGLApp {
protected:
	TextureID tex0, tex1, tex2;
	TextureID distanceMap, renderTexture;

	ShaderID fragmentShader;

    MaterialTable matTable;
	float dof;

	Set <class Polygon *> polys;

	void drawWorld();

public:
	void selectPixelFormat(PixelFormat &pixelFormat);

	bool init();
	bool exit();
	bool load();
	bool unload();
	bool drawFrame();
	
	void resetCamera();
};

void MainApp::resetCamera(){
	position = Vertex(-150, -100, 600);
	Wx = 0;
	Wy = 3.3f;
	Wz = 0;
}

void MainApp::selectPixelFormat(PixelFormat &pixelFormat){
	pixelFormat.redBits   = 8;
	pixelFormat.greenBits = 8;
	pixelFormat.blueBits  = 8;
	pixelFormat.alphaBits = 0;

	pixelFormat.depthBits = 16;
	pixelFormat.stencilBits = 0;
}


bool polygonFilter(const int flags){
	return ((flags & PF_Invisible) == 0) && ((flags & PF_TwoSided) == 0) && (flags >= 0);
}

bool MainApp::init(){
	dof = 0.5f;
	speed = 512;

	return true;
}

bool MainApp::exit(){

	return true;
}

bool MainApp::load(){

	String supported, missing;

    if (GL_ARB_fragment_program)   supported += "GL_ARB_fragment_program\n";   else missing += "GL_ARB_fragment_program\n";
    if (GL_SGIS_generate_mipmap)   supported += "GL_SGIS_generate_mipmap\n";   else missing += "GL_SGIS_generate_mipmap\n";
    if (GL_EXT_texture3D)          supported += "GL_EXT_texture3D\n";          else missing += "GL_EXT_texture3D\n";
    if (GL_EXT_texture_edge_clamp) supported += "GL_EXT_texture_edge_clamp\n"; else missing += "GL_EXT_texture_edge_clamp\n";
#ifdef _WIN32
	if (WGL_ARB_render_texture)    supported += "WGL_ARB_render_texture\n";    else missing += "WGL_ARB_render_texture\n";
#else
	if (GLX_ATI_render_texture)    supported += "GLX_ATI_render_texture\n";    else missing += "GLX_ATI_render_texture\n";
#endif

    if (missing.getLength() > 0){
        ALERT("The following required OpenGL extensions are supported:\n\n" + supported +
			  "\nToo bad, these required OpenGL extensions aren't supported by the graphic card and driver you're using:\n\n" + missing +
              "\nThis application will now exit.", "Missing extensions");
        return false;
    }
	
	projection.loadProjectionMatrix(90 * PI / 180, 0.75f, 1, 2000);

	distanceMap = renderer->addDistanceMap(64, TEX_NOMIPMAPPING);

	renderTexture  = renderer->addRenderTexture(1024, 1024, true, false, RPF_RGBA8);

	tex0 = renderer->addTexture("../Textures/matta.jpg");
	tex1 = renderer->addTexture("../Textures/Wood.jpg");
	tex2 = renderer->addTexture("../Textures/wooddecoration1.dds");

	textTexture = renderer->addTexture("../Textures/font2.dds");

	fragmentShader = renderer->addFragmentShader(GL_FRAGMENT_PROGRAM_ARB, "dof.fsh");

	if (glGetError() != GL_NO_ERROR){
		char *str = (char *) glGetString(GL_PROGRAM_ERROR_STRING_ARB);
		ALERT(str, "Error");
	}


	Material *mat;

	mat = new Material();
	mat->setTextures(tex0);
    matTable.addMaterialName("EMarble5", 256, 256, mat, 0);

	mat = new Material();
	mat->setTextures(tex1);
    matTable.addMaterialName("EMarble11", 256, 256, mat, 0);

	mat = new Material();
	mat->setTextures(tex1);
    matTable.addMaterialName("EMarble11b", 256, 256, mat, 0);

	mat = new Material();
	mat->setTextures(tex1);
    matTable.addMaterialName("erail", 256, 256, mat, 0);

	mat = new Material();
	mat->setTextures(tex1);
    matTable.addMaterialName("eredcarpet", 256, 256, mat, 0);

	mat = new Material();
	mat->setTextures(tex2);
    matTable.addMaterialName("EPanel", 256, 256, mat, 0);
	
	mat = new Material();
	mat->setTextures(tex2);
    matTable.addMaterialName("inlaid7", 256, 256, mat, 0);

	mat = new Material();
	mat->setTextures(tex2);
    matTable.addMaterialName("EStone23", 256, 256, mat, 0);
	
	T3dLoader t3dLoader;
	t3dLoader.loadFromFile("trophyroom.t3d", polys, matTable, depthPolyCreator, polygonFilter, Vertex(0,0,0));

	return true;
}

bool MainApp::unload(){
	polys.clear();
	matTable.clear();
	return true;
}

void MainApp::drawWorld(){

	glColor3f(1,1,1);

	for (unsigned int j = 0; j < polys.getSize(); j++){
		((DepthPolygon *) polys[j])->draw(renderer, position, 4000);
	}
}


bool MainApp::drawFrame(){
#ifdef WIN32
	if (keys[VK_ADD]){
#else
	if (keys[XK_KP_Add]){
#endif
		dof -= time;
		if (dof < 0) dof = 0;
	}
#ifdef WIN32
	if (keys[VK_SUBTRACT]){
#else
	if (keys[XK_KP_Subtract]){
#endif
		dof += time;
		if (dof > 1) dof = 1;
	}



	renderer->changeRenderTarget(renderTexture);

	//glClearColor(1,0,0,0);
	glClear(/*GL_COLOR_BUFFER_BIT | */GL_DEPTH_BUFFER_BIT);

	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);

	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(projection);

	glMatrixMode(GL_MODELVIEW);
	modelView.loadRotateZXY(-Wx, -Wy, -Wz);
	modelView.translate(-position);
	glLoadMatrixf(modelView);

	drawWorld();

	renderer->changeRenderTarget();


	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0,1,0,1,-1,1);


	glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 0, dof, 0, 0, 0);


	renderer->setFragmentShader(fragmentShader);
	renderer->setTextures(renderTexture);
	renderer->apply();

	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2f(0,0);
		glTexCoord2f(1,0);
		glVertex2f(1,0);
		glTexCoord2f(1,1);
		glVertex2f(1,1);
		glTexCoord2f(0,1);
		glVertex2f(0,1);
	glEnd();

	return true;
}

DECLARE_MAIN_APP(MainApp)
